%% Initial comments

% This example program is provided as is and without support.   Keysight is
% not responsible for modifications. 

%"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
% Copyright  2015 Keysight Technologies Inc. All rights reserved.
%
% You have a royalty-free right to use, modify, reproduce and distribute this
% example files (and/or any modified version) in any way you find useful, provided
% that you agree that Keysight has no warranty, obligations or liability for any
% Sample Application Files.
%
%"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

% This script shows how to properly initialize a Keysight InfiniiVision or
% InfiniiVisionX X series scope, and synchronize an acquisition using the
% blocking :DIGitize command.  This method DOES work for average
% acquisition mode AND math functions.  At the very end of the script is more detail on how
% this works, the benefits and drawbacks of this method.

% This script should work for all InfiniiVision and InfiniiVision-X oscilloscopes:
% DSO5000A, DSO/MSO6000A/L, DSO/MSO7000A/B, DSO/MSO-X2000A, DSO/MSO-X3000A/T, DSO/MSO-X4000A, DSO/MSO-X6000A

% This example requires that you have the MATLAB instrument control toolbox.
% You can find more about it by visiting the Mathworks website at: 
% http://www.mathworks.com/products/instrument/ 

%% Clear MATLAB

% Clear everything
clear 
clear all
clc
close
close all

% Find all previously connected instruments and close them
newobjs = instrfind;  
if isempty(newobjs) == false;
    fclose(newobjs);
    delete(newobjs);
end
clear newobjs

%% User defined constants
GLOBAL_TIMEOUT = 10;
SCOPE_VISA_ADDRESS = 'USB0::0x0957::0x1783::MY47050006::0::INSTR';
TIME_TO_TRIGGER = 10; % Time in seconds
    % This is the time until the FIRST trigger event.
    % While the script calculates a general time out for the given setup, it cannot know when a trigger event will occur.  Thus, the user must still set this value.
    % This time is in addition to the calculated minimum timeout... so, if a scope might take say, 1 us to arm and acquire data,
        % the signal might take 100 seconds before it occurs... this accounts for that.
    % The SCOPE_ACQUISITION_TIME_OUT calculation pads this by 1.1

TIME_BETWEEN_TRIGGERS = 0.25; % Time in seconds - for Average, Segmented, and Equivalent Time modes, else set to 0
    % In Average, Segmented, and Equivalent Time modes, the scope makes repeated acquisitions.  This is similar to 
        % the above TIME_TO_TRIGGER, but it is the time BETWEEN triggers.  For example, it might take 10 seconds 
        % for the first trigger event, and then they might start occurring regularly at say, 1 ms intervals.  In 
        % that scenario, 15 seconds (a conservative number for 10s) would be good for TIME_TO_TRIGGER,
        % and 2 ms (again conservative) would be good for TIME_BETWEEN_TRIGGERS.
    % The default in this sample script is 0.025 seconds. This is to make the sample work for the LINE trigger
        % used in this script when the scope is in Average, Segmented, and Equivalent Time modes to force a
        % trigger off of the AC input line (:TRIGger:EDGE:SOURce LINE) which runs at 50 or 60 Hz in most
        % of the world (1/50 Hz -> 20 ms, so use 25 ms to be conservative).
    % The SCOPE_ACQUISITION_TIME_OUT calculation pads this by 1.1


%% Connect and Initialize Scope

% Define the oscilloscope via the MATHWORKS Instument Control Toolbox
KsInfiniiVisionX = visa('agilent', SCOPE_VISA_ADDRESS); % At some point, 
% one may need to change 'agilent' to 'keysight' if MATHWORKS and/or 
% Keysight ever makes that change.

% Set global timeout for the scope; can rest this as needed for specific
% commands (before and after)
KsInfiniiVisionX.Timeout = GLOBAL_TIMEOUT;

% Connect to (open) the scope
fopen(KsInfiniiVisionX);

% Clear the instrument's bus
clrdevice(KsInfiniiVisionX);

% Reset the scope if desired
% fprintf(KsInfiniiVisionX, '*RST');

% Clear the scope registers; Always stop scope before setting it up; wait
% for *OPC to come back. Always do a :STOP as a :STOP;*OPC? query
query(KsInfiniiVisionX, '*CLS;STOP;*OPC?');

% Scope is connected and initialized; ready to accept commands or do
% something

%% Scope Setup

% Reset the scope if desired - not done for this example
% fprintf(KsInfiniiVisionX, '*RST');
% Insert scope setup here as desired

% For this example, the scope will be forced to trigger on the (power) LINE voltage so something happens
fprintf(KsInfiniiVisionX, ':TRIGger:SWEep NORMal'); 
query(KsInfiniiVisionX, ':TRIGger:EDGE:SOURce LINE;*OPC?'); % This line simply gives the scope something to trigger on

% End Scope Setup

%% Calculate acquisition timeout/wait time by short, overestimate method

% Create some default variables
N_AVERAGES = 1;
N_SEGMENTS = 1;

% Get some info about the scope time base setup
TimeRange    = str2double(query(KsInfiniiVisionX,':TIMebase:RANGe?')); % Note this is 10x the time SCALE
TimePosition = str2double(query(KsInfiniiVisionX,':TIMebase:POSition?'));
TrigHoldOff  = str2double(query(KsInfiniiVisionX,':TRIGger:HOLDoff?'));

% Determine Acquisition Type and Mode:
ACQ_TYPE = query(KsInfiniiVisionX, ':ACQuire:TYPE?');
ACQ_MODE = str2double(query(KsInfiniiVisionX,':ACQuire:MODE?'));

if strcmp(ACQ_MODE,'SEGM');
    N_SEGMENTS = query(KsInfiniiVisionX,':ACQuire:SEGMented:COUNt?');
    % Note that if there is a lot of analysis associated segments, e.g. serial data decode, the timeout will likely need to be longer than calculated.
        % The user is encouraged to manually set up the scope in this case, as it will be used, and time it, and use that, with a little overhead.
        % Blocking method is recommended for Segmented Memory mode.
elseif strcmp(ACQ_TYPE,'AVER');
    N_AVERAGES = query(KsInfiniiVisionX,':ACQuire:COUNt?');
end

% Determine if Sgemented Memory or Equivalent time modes are on
ACQ_MODE = str2double(query(KsInfiniiVisionX,':ACQuire:MODE?'));
ETIME_MULTIPLIER = 1;

% Calculate acuisition timeout by overestimate method:
SCOPE_ACQUISITION_TIME_OUT = (TIME_TO_TRIGGER)*1.1 + (TimeRange*2 + abs(TimePosition)*2 + TrigHoldOff*1.1 + TIME_BETWEEN_TRIGGERS*1.1)*N_SEGMENTS*N_AVERAGES;

% Ensure the timeout is no less than 10 seconds
if SCOPE_ACQUISITION_TIME_OUT < 10;
    SCOPE_ACQUISITION_TIME_OUT = 10;
end

% What about Equivalent Time Mode and other odd modes such as Jitter or Eye (the last two only being found on the X6000A), and math functions?
        % In most cases, the padding and 10 second minimum timeout will take care of this.
        % Equivalent Time Mode only has an effects at the fastest time scales, so it really doesnt make a difference as long as a trigger signal is present. If trigger signal occurs rarely, adjust the TIME_BETWEEN_TRIGGERS constant accordingly.
        % For math, the math will happen fast enough that the padding in the timeout calculation takes care of this.
        % For jitter mode on the X6000A, the user can try this, method, and typically there is always s signal present, and the 10 second minimum should work out.  If not, make it bigger, or increase padding.
        % For Eye mode on the X6000A, none of this works anyway, and you have to use :RUN (or :RTEYe:ACQuire) and :STOP.

%% Acquire signal on scope

% Choose either InfiniiVision_blocking_method or InfiniiVision_polling_method
% There is no a-priori reason to do this as a function except that a user would probably want to use it repeatedly

% If Acquistion mode is Average, alwayss use blocking_method() to get
% complete average.

%[Time_to_Acquire Signal_Acquired] = InfiniiVision_blocking_method(KsInfiniiVisionX,SCOPE_ACQUISITION_TIME_OUT,GLOBAL_TIMEOUT);
% OR

[Time_to_Acquire Signal_Acquired] = InfiniiVision_polling_method(KsInfiniiVisionX,SCOPE_ACQUISITION_TIME_OUT);

if Signal_Acquired == 0;
    % Throw an MException, in effect treating the warning as an error
    % This WILL say that there is a problem with the code, but it's not.
    % In the condition of signal not acquired, we should close the script,
    % or potentially take some other action, e.g. force a trigger.
    disp 'The next thing in the Command Window sure looks like an error message.  It is, but not because there is a bug'
    disp 'in the scpript.  We MADE THIS ERROR to handle an acquisition that failed!'
    TimeOutExeption = MException('myscript:error','Acquisition timed out due to no trigger, or improper setup causing no trigger, or too short of a time out.');
    throw(TimeOutExeption);
end

fprintf('It took %d seconds to acquire the signal.\n',Time_to_Acquire);

% Done Acquire signal on scope
%% Do something

Vpp = str2double(query(KsInfiniiVisionX, ':MEASure:VPP?'));
fprintf('Vpp = %d V.\n',Vpp);

%% Properly close scope connection

fclose(KsInfiniiVisionX);
delete(KsInfiniiVisionX);
clear KsInfiniiVisionX;

disp 'Done.'