function [time, Acquired] = InfiniiVision_blocking_method( KsIvScope, KsIvScope_Acq_timeout, global_timeout)
%Define a simple and fast function utilizing the blocking :DIGitize command
%in conjunction with *OPC? to acquire signals on the scope
% Benefits of this method:
    % Fastest, compact
    % Only way for Average Acquisition mode:
        % The :SINGle command does not do a complete average.
        % Counting triggers with :RUN is much too slow
    % Allows for easy synchronization with scope math functions
    % Don't have to deal with the status registers, which can be confusing
    % Potentially faster than polling_method(), for better trhoughput
    % Because it's faster one can retreive more accurate acquisition times
    % than with a polling method.
% Drawbacks of this method:
    % Requires a well-chosen, hard-set timeout that will cover the time to arm, trigger,
        % and finish acquisition.
    % Requires Exception handling and a device_clear() for a possible timeout (no trigger event).
        % Socket connectiosn cannot do device_clear()
    % Since :DIGitize is a "specialized form of the :RUN" command, on these scope, that results in:
        % the sample rate MAY be reduced from using :SINGle - usually at longer time scales - 
        % typically only acquires what is on screen, though at the fastest time scales, more than on screen data may be acquired
        % Thus, for max memory and max sample rate, use the polling_method(), which uses :SINGle.
% How it works:
    % The :DIGitize command is a blocking command, and thus, all other SCPI commands are blocked until
        % :DIGitize is completely done.  This includes any subsequent processing that is already set up,
        % such as math, jitter separation, and measurements.  Key Point: When the *OPC? query is appended to
        % :DIGitize with a semi-colon (;), which essentially ties it to the same thread in the parser,
        % it is immediately dealt with when :DIGitize finishes and gives a 1 back to the script, allowing the script to move on.
% Other Notes:
    % If you DO NOT know when a trigger will occur, you will need to (should) set a very long time out.
    % The timeout will need to be (should be) adjusted before and after the :DIGitize operation,
        % though this is not absolutely required.
    % A :DIGitize can be aborted with a device clear:
        % clrdevice(KsInfiniiVisionX); or in this function clrdevice(scope)
    % :DIGItize disables the anti-aliasing feature (sample rate dither) on all InfiniiVision and InfiniiVision-X scopes.
    % :DIGitize temporarily locks the front panel, and all front panel presses are queued until :DIGitize is done.  So if you change the vertical scale, it will not happen until the acquisition is done.

    % Turn off time out warning since this can be confusing in this
    % particular context.  We will catch thsi warning, and treat it as an
    % error in the main script.  Be sure to turn this back on after as you
    % are developing the code.
    warning('off','instrument:query:unsuccessfulRead')

    disp 'Acquiring signal(s)...'
    KsIvScope.Timeout = KsIvScope_Acq_timeout;% Note that this is a property of the 
    % device interface, KsIvScope.
        % If doing repeated acquisitions, this should be done BEFORE the loop,
        % and changed again after the loop if the goal is to achieve best throughput.

    lastwarn('', ''); % Reset the lastwarn to null, to properly handle a possible timeout

    tic;
    query(KsIvScope, ':DIGitize;*OPC?'); % Acquire the signal(s) with :DIGItize 
    % (blocking) and wait until *OPC? comes back with a one.

     % Note that this DOES NOT work with :SINGle
     % This is a common misconception.  :SINGle is NOT blocking, and thus the
     % "1" comes back as soon as the :SINGle command is doen being parsed by
     % the scope. With a blocking command, it does not come back until the
     % blocking command is done.

    % Check for and handle a timeout
    [warnStr, warnId] = lastwarn; % Get last warning 
    if strcmpi(warnStr, 'Unsuccessful read: VISA: A timeout occurred');
        % If you are here, then a timeout occurred on read, indicating the
        % acquistion did not finish in the time out

        disp 'Acquisition timed out.'
        disp 'Properly closing scope and aborting script.'
        clrdevice(KsIvScope) % clear scope; unlocks the bus as :DIGitize is blocking
        fprintf(KsIvScope, '*CLS');
        % disconnect from scope
        fclose(KsIvScope);
        delete(KsIvScope);
        clear KsIvScope;
        Acquired = 0;
        time = 0;
    else    
        KsIvScope.Timeout = global_timeout; %  Reset timeout back to what it was, global_timeout
        time = toc;
        disp 'Signal(s) acquired!'    
        Acquired = 1;
    end
    % Turn time out warnings back on
    warning('on','instrument:query:unsuccessfulRead')
end

