function [ time, Acquired ] = InfiniiVision_polling_method (  KsIvScope, KsIvScope_Acq_timeout )
%Define a function using the non-blocking :SINGle command and polling on
%the Operation Condition register to acquire signals on the scope
% Benefits of this method:
    % Don't have to worry about interface timeouts
    % Easy to expand to know when scope is armed, and triggered
    % MAY result in a higher sample rate thanteh blocking method
    % Always fills max available memory
% Drawbacks of this method:
    % Slow
    % Does NOT work for Average Acquisition mode
        % :SINGle does not do a complete average, and counting triggers in :RUN is much too slow
            % It does a single acquisition as if it were in NORMal acq. mode
            % Counting triggers in :RUN is much too slow
    % Can't be used effectively for synchronizing math functions
        % It can be done by applying an additional hard coded wait after the acquisition is done.  At least 200 ms is suggested, more may be required.
        % However, as long as the time out is not excessively short, the math happens fast enough that once :OPERegister:CONDition? comes back as done
            % that one can just wait for it.  The exception would be for eye or jitter mode on an X6000A, where the processing time can be long.
    % Still need some maximum timeout (here MAX_TIME_TO_WAIT), ideally, or the script will sit in the while loop forever if there is no trigger event
        % Max time out (here MAX_TIME_TO_WAIT) must also account for any processing done(see comments on amth above)
        % Max time out (here MAX_TIME_TO_WAIT) must also account for time to arm the scope and finish the acquisition
            % This arm/trigger/finsih part is accounted for in the main script.
% How it works:
    % Pretty well explained in line; see annotated screenshots. Basically:
        % What really matters is the RUN bit in the Operation Condition (not Event) Register.  This bit changes based on the scope state.
        % If the scope is running, it is high (8), and low (0) if it is stopped.
        % The only (best) way to get at this bit is with the :OPERation:CONDition? query.  The Operation Condition Register can reflect states
        % for other scope properties, for example, if the scope is armed, thus it can produce values other than 0 (stopped) or 8 (running).
        % To handle that, the result of :OPERation:Condition? is bitwise ANDed with an 8.  This is called "unmasking." 
        % Here, the "unmasking" is done in the script.  On the other hand, it is possible to "mask" which bits get passed to the
        % summary bit to the next register below on the instrument itself.  However, this method it typically only used when working with the Status Byte,
        % and not used here.
        % Why 8 = running = not done?
            % The Run bit is the 4th bit of the Operation Status Condition (and Event) Registers.
            % The registers are binary and start counting at zero, thus the 4th bit is bit number 3, and 2^3 = 8, and thus it returns an 8 for high and a 0 for low.
        % Why the CONDITION and NOT the EVENT register?
            % The Condition register reflects the CURRENT state, while the EVENT register reflects the first event that occurred since it was cleared or read,
            % thus the CONDTION register is used.
        

    MAX_TIME_TO_WAIT = KsIvScope_Acq_timeout; % Time in seconds to wait for the scope to arm, trigger, and finish acquisition.
        % Note that this is NOT a property of the device interface, 
        % KsIvScope, but rather some constant in the script to be used later
        % with a timer.
    
    % Define "mask" bits and completion criterion.
     % Mask bits for Run state in the Operation Status Condition (and Event) Register
         % This can be confusing.  In general, refer to Programmer's Guide chapters on Status Reporting, and Synchronizing Acquisitions
         % Also see the annotated screenshots included with this sample script.
    RUN_BIT  = 3; % The run bit is the 4th bit (see next set of comments @ Compeltion Criteria).
    RUN_MASK = 2^RUN_BIT; % This basically means:  2^3 = 8; this is used later to
        % "unmask" the result of the Operation Status Event Register as there is no direct access to the RUN bit.

    % Completion criteria
    ACQ_DONE = 0; % Means the scope is stopped
    ACQ_NOT_DONE = 8; % Means the scope is running; value is 8
        %This is the 4th bit of the Operation Status Condition (and Event) Register.  
        % he registers are binary and start counting at zero, thus the 4th bit (4th postion in a binary representaion of decimal 8 = 2^3
        % This is either High (running = 8) or low (stopped and therefore done with acquisition = 0).
    
    disp 'Acquiring signal(s)...'
    StartTime = tic; %Define acquisition start time; This is in seconds.
    fprintf(KsIvScope, '*CLS;SINGle'); % Beigin Acquisition with *CLS and the non-blocking :SINGle command, concatenated together. The *CLS clears all (non-mask) registers & sets them to 0;
    
    % Initialize the loop entry condition (assume Acq. is not done).
    Acq_State = ACQ_NOT_DONE;
    
    % Poll the scope until Acq_State is a one. (This is NOT a "Serial Poll.")
    while Acq_State == ACQ_NOT_DONE && (toc(StartTime) <= MAX_TIME_TO_WAIT); % # This loop is never entered if the acquisition completes immediately; Exits if Status == 8 ( 8 & 8 = 8) or MAX_TIME_TO_WAIT exceeded
        Status = str2double(query(KsIvScope, ':OPERegister:CONDition?')); % Ask scope if it is done with the acquisition via the Operation Status Condition (not Event) Register.
        Acq_State = bitand(Status,RUN_MASK);
        if Acq_State == ACQ_DONE;
            break % Break out of while loop so that the 100 ms pause below is not incurred if done.
        end
        pause(0.1); % Pause 100 ms to prevent excessive queries
        % This can actually be set a little faster, at 0.045.  The point here is that
            % 1. if there are other things being cotnrolled, going too fast can tie up the bus.
            % 2. going faster does not work on all scopes. The symptom of this not working is:
                    % The scope SHOULD trigger, but it sits there with the Single hark key on the scope lit yellow; hitting this key causes a trigger.
        % Loop exists when Status != NOT_DONE, that is, it exits the loop when it is DONE
    end
    
    
    if Acq_State == ACQ_DONE; % Acquisition fully completed
        disp 'Signal acquired.'
        Acquired = 1;
        time = toc(StartTime); % Due to the necessary wait times, this is not too accurate
    else % Acquisition failed for some reason
        disp 'Acquisition timed out.'
        disp 'Properly closing scope and aborting script.'
        query(KsInfiniiVisionX, ':STOP;*OPC?'); % stop scope
        clrdevice(KsIvScope); % clear scope communications interface
        % Don't do  *CLS.  If you do, you won't be able to do a meaningfull :SYSTem:ERRor? query as *CLS clears the error queue
        % disconnect from scope
        fclose(KsIvScope);
        delete(KsIvScope);
        clear KsIvScope;
        Acquired = 0;
        time = 0;
    end

end